@@ -34,7 +34,21 @@ test_that("'mod_pois' works with valid inputs - no exposure", {
     formula <- deaths ~ age:sex + time
     ans_obtained <- mod_pois(formula = formula,
                              data = data,
-                             exposure = 1)
+                             exposure = NULL)
+    expect_s3_class(ans_obtained, "bage_mod_pois")
+    ans_obtained <- mod_pois(formula = formula,
+                             data = data,
+                             exposure = NULL)
+    expect_s3_class(ans_obtained, "bage_mod_pois")
+})
+
+test_that("'mod_pois' works with valid inputs - no response variable", {
+    data <- expand.grid(age = 0:2, time = 2000:2001, sex = 1:2)
+    data$popn <- seq_len(nrow(data))
+    formula <- deaths ~ age:sex + time
+    ans_obtained <- mod_pois(formula = formula,
+                             data = data,
+                             exposure = NULL)
     expect_s3_class(ans_obtained, "bage_mod_pois")
 })
 
@@ -49,7 +63,6 @@ test_that("'mod_pois' gives correct error when no exposure specified", {
 })
 
 
-
 ## 'mod_binom' ----------------------------------------------------------------
 
 test_that("'mod_binom' works with valid inputs", {
@@ -73,6 +86,17 @@ test_that("'mod_binom' works with valid inputs", {
     expect_identical(ans_noquote, ans_withquote)
 })
 
+test_that("'mod_binom' works with no response", {
+    data <- expand.grid(age = 0:2, time = 2000:2001, sex = 1:2)
+    data$popn <- seq_len(nrow(data))
+    formula <- deaths ~ age:sex + time
+    ans_obtained <- mod_binom(formula = formula,
+                              data = data,
+                              size = popn)
+    expect_s3_class(ans_obtained, "bage_mod_binom")
+    expect_s3_class(ans_obtained, "bage_mod")
+})
+
 test_that("'mod_binom' gives correct error when no size supplied", {
     data <- expand.grid(age = 0:2, time = 2000:2001, sex = 1:2)
     data$popn <- seq_len(nrow(data))
@@ -84,7 +108,6 @@ test_that("'mod_binom' gives correct error when no size supplied", {
 })
 
 
-
 ## 'mod_norm' -----------------------------------------------------------------
 
 test_that("'mod_norm' works with valid inputs - with weights", {
@@ -109,13 +132,28 @@ test_that("'mod_norm' works with valid inputs - with weights", {
 })
 
 test_that("'mod_norm' works with valid inputs - no weights", {
+  data <- expand.grid(age = 0:2, time = 2000:2001, sex = 1:2)
+  data$popn <- seq_len(nrow(data))
+  data$income <- rnorm(nrow(data))
+  formula <- income ~ age:sex + time
+  ans_obtained <- mod_norm(formula = formula,
+                           data = data)
+  expect_s3_class(ans_obtained, "bage_mod_norm")
+  ans_obtained <- mod_norm(formula = formula,
+                           data = data,
+                           weights = NULL)
+  expect_s3_class(ans_obtained, "bage_mod_norm")
+  ans_obtained <- mod_norm(formula = formula,
+                           data = data)
+  expect_s3_class(ans_obtained, "bage_mod_norm")
+})
+
+test_that("'mod_norm' works with valid inputs - no response", {
     data <- expand.grid(age = 0:2, time = 2000:2001, sex = 1:2)
     data$popn <- seq_len(nrow(data))
-    data$income <- rnorm(nrow(data))
     formula <- income ~ age:sex + time
     ans_obtained <- mod_norm(formula = formula,
-                             data = data,
-                             weights = 1)
+                             data = data)
     expect_s3_class(ans_obtained, "bage_mod_norm")
 })
 
@@ -125,23 +163,12 @@ test_that("'mod_norm' works with no values for outcome variable", {
     data$income <- NA
     formula <- income ~ age:sex + time
     ans_obtained <- mod_norm(formula = formula,
-                             data = data,
-                             weights = 1)
+                             data = data)
     expect_s3_class(ans_obtained, "bage_mod_norm")
     expect_identical(ans_obtained$outcome_mean, 0)
     expect_identical(ans_obtained$outcome_sd, 1)
 })
 
-test_that("'mod_norm' gives correct error when no weights supplied", {
-    data <- expand.grid(age = 0:2, time = 2000:2001, sex = 1:2)
-    data$popn <- seq_len(nrow(data))
-    data$income <- rnorm(nrow(data))
-    formula <- income ~ age:sex + time
-    expect_error(mod_norm(formula = formula,
-                          data = data),
-                 "Argument `weights` is missing, with no default.")
-})
-
 
 ## 'mod_helper' ---------------------------------------------------------------
 
