@@ -180,75 +180,6 @@ test_that("'dimnames_to_nm_split' works with 2D dimnames", {
 })
 
 
-## 'error_offset_formula_used' ------------------------------------------------
-
-test_that("'error_offset_formula_used' returns TRUE with non-formula", {
-  expect_true(error_offset_formula_used(nm_offset_data = "popn",
-                                        nm_offset_mod = "exposure",
-                                        nm_fun = "forecast"))
-})
-
-test_that("'error_offset_formula_used' raises expected error with formula, Poisson", {
-  expect_error(error_offset_formula_used(nm_offset_data = "~popn",
-                                        nm_offset_mod = "exposure",
-                                        nm_fun = "forecast"),
-               "`forecast\\(\\)` cannot be used with models where exposure specified using formula.")
-})
-
-test_that("'error_offset_formula_used' raises expected error with formula, binomial", {
-  expect_error(error_offset_formula_used(nm_offset_data = "~popn",
-                                        nm_offset_mod = "size",
-                                        nm_fun = "forecast"),
-               "`forecast\\(\\)` cannot be used with models where size specified using formula.")
-})
-
-test_that("'error_offset_formula_used' raises expected error with formula, normal", {
-  expect_error(error_offset_formula_used(nm_offset_data = "~popn",
-                                        nm_offset_mod = "weights",
-                                        nm_fun = "forecast"),
-               "`forecast\\(\\)` cannot be used with models where weights specified using formula.")
-})
-
-test_that("'error_offset_formula_used' throws error with invalid nm_offset_mod", {
-  expect_error(error_offset_formula_used(nm_offset_data = "~popn",
-                                        nm_offset_mod = "wrong",
-                                        nm_fun = "forecast"),
-               "Internal error")
-})
-
-
-
-
-
-
-## 'eval_offset_formula' ------------------------------------------------------
-
-test_that("'eval_offset_formula' works with valid inputs - simple formula", {
-  nm_offset_data <- "~popn + other"
-  data <- data.frame(popn = 1, other = 2)
-  ans_obtained <- eval_offset_formula(nm_offset_data = nm_offset_data, data = data)
-  ans_expected <- 3
-  expect_identical(ans_obtained, ans_expected)
-})
-
-test_that("'eval_offset_formula' works with valid inputs - complicated formula", {
-  nm_offset_data <- "~popn^2 + log(other) + 6"
-  data <- data.frame(popn = 1:2, other = 2:3)
-  ans_obtained <- eval_offset_formula(nm_offset_data = nm_offset_data, data = data)
-  ans_expected <- (1:2)^2 + log(2:3) + 6
-  expect_identical(ans_obtained, ans_expected)
-})
-
-
-test_that("'eval_offset_formula' works with valid inputs - ifelse", {
-  nm_offset_data <- "~ifelse(popn <= 0, 0.1, popn)"
-  data <- data.frame(popn = 0:2)
-  ans_obtained <- eval_offset_formula(nm_offset_data = nm_offset_data, data = data)
-  ans_expected <- c(0.1, 1, 2)
-  expect_identical(ans_obtained, ans_expected)
-})
-
-
 ## 'get_matrix_offset_svd_prior' ----------------------------------------------
 
 
@@ -293,7 +224,7 @@ test_that("'get_matrix_or_offset_svd_prior' works with sex-age interaction, type
 
 test_that("'get_matrix_or_offset_svd_prior' works with age-sex interaction, type is indep, matrix", {
   ssvd <- sim_ssvd()
-  prior <- SVD(ssvd)
+  prior <- SVD(ssvd, n_comp = 5)
   dimnames_term <- list(sex = c("Female", "Male"),
                         age = c("0-4", "5-9"))
   ans_obtained <- get_matrix_or_offset_svd_prior(prior,
@@ -563,6 +494,20 @@ test_that("'init_val_sd' works", {
 })
 
 
+## 'is_prior_ok_for_term_along' -----------------------------------------------
+
+test_that("'is_prior_ok_for_term_along' works", {
+  expect_true(
+    is_prior_ok_for_term_along(prior = AR1(),
+                               min_length_along = 2L,
+                               dimnames_term = list(time = 2001:2004),
+                               var_time = "time",
+                               var_age = "age",
+                               var_sexgender = "sex")
+  )
+})
+
+
 ## 'make_agesex' --------------------------------------------------------------
 
 test_that("'make_agesex' works with valid inputs", {
@@ -887,6 +832,24 @@ test_that("'make_hyperrandfree' works with valid inputs - has hyperrandfree", {
 })
 
 
+## 'make_i_along_has_along' ---------------------------------------------------
+
+test_that("'make_i_along_has_along' works", {
+  prior <- AR()
+  dimnames_term <- list(age = 1:3,
+                        time = 2000:2005,
+                        sex = c("f", "m"))
+  var_time <- "time"
+  var_age <- "age"
+  ans_obtained <- make_i_along_has_along(prior = prior,
+                                         dimnames_term = dimnames_term,
+                                         var_time = var_time,
+                                         var_age = var_age)
+  ans_expected <- 2L
+  expect_identical(ans_obtained, ans_expected)
+})
+
+
 ## 'make_i_prior' -------------------------------------------------------------
 
 test_that("'make_i_prior' works with valid inputs", {
@@ -1155,7 +1118,7 @@ test_that("'make_map_effectfree_fixed' works with valid inputs", {
 })
 
 
-## 'make_matrices_along_by_effectfree' ------------------------------------------------
+## 'make_matrices_along_by_effectfree' ----------------------------------------
 
 test_that("'make_matrices_along_by_effectfree' works", {
   set.seed(0)
@@ -1316,7 +1279,7 @@ test_that("'make_matrices_effectfree_effect' works with valid inputs", {
 
 ## 'make_matrix_covariates' ---------------------------------------------------
 
-test_that("'make_matrix_covariates' works with valid inputs - all numeric", {
+test_that("'make_matrix_covariates' works with valid inputs - all numeric - rows NULL", {
   set.seed(0)
   data <- expand.grid(age = 0:9,
                       region = c("a", "b"),
@@ -1326,7 +1289,7 @@ test_that("'make_matrix_covariates' works with valid inputs - all numeric", {
   data$income <- runif(n = nrow(data))
   data$distance <- runif(n = nrow(data))
   formula <- ~ income + distance
-  ans_obtained <- make_matrix_covariates(formula = formula, data = data)
+  ans_obtained <- make_matrix_covariates(formula = formula, data = data, rows = NULL)
   ans_expected <- Matrix::sparse.model.matrix(~income + distance - 1,
                                               data = data)
   ans_expected[,"income"] <- scale(ans_expected[,"income"])
@@ -1337,7 +1300,29 @@ test_that("'make_matrix_covariates' works with valid inputs - all numeric", {
   expect_equal(ans_obtained, ans_expected)
 })
 
-test_that("'make_matrix_covariates' works with valid inputs - not all numeric - has intercept", {
+test_that("'make_matrix_covariates' works with valid inputs - all numeric - rows non-NULL", {
+  set.seed(0)
+  data <- expand.grid(age = 0:9,
+                      region = c("a", "b"),
+                      sex = c("F", "M"))
+  data$popn <- rpois(n = nrow(data), lambda = 100)
+  data$deaths <- rpois(n = nrow(data), lambda = 10)
+  data$income <- runif(n = nrow(data))
+  data$distance <- runif(n = nrow(data))
+  formula <- ~ income + distance
+  ans_obtained <- make_matrix_covariates(formula = formula, data = data, rows = 35:40)
+  ans_expected <- Matrix::sparse.model.matrix(~income + distance - 1,
+                                              data = data)
+  ans_expected[,"income"] <- scale(ans_expected[,"income"])
+  ans_expected[,"distance"] <- scale(ans_expected[,"distance"])
+  rownames(ans_expected) <- NULL
+  attributes(ans_expected)$assign <- NULL
+  rownames(ans_expected) <- NULL
+  ans_expected <- ans_expected[35:40,]
+  expect_equal(ans_obtained, ans_expected)
+})
+
+test_that("'make_matrix_covariates' works with valid inputs - not all numeric - has intercept, rows NULL", {
   set.seed(0)
   data <- expand.grid(age = 0:9,
                       region = c("a", "b"),
@@ -1346,7 +1331,7 @@ test_that("'make_matrix_covariates' works with valid inputs - not all numeric -
   data$deaths <- rpois(n = nrow(data), lambda = 10)
   data$income <- runif(n = nrow(data))
   formula <- ~ income * region
-  ans_obtained <- make_matrix_covariates(formula = formula, data = data)
+  ans_obtained <- make_matrix_covariates(formula = formula, data = data, rows = NULL)
   data_scaled <- data
   data_scaled$income <- as.numeric(scale(data_scaled$income))
   ans_expected <- Matrix::sparse.model.matrix(~income*region,
@@ -1355,7 +1340,26 @@ test_that("'make_matrix_covariates' works with valid inputs - not all numeric -
   expect_identical(ans_obtained, ans_expected)
 })
 
-test_that("'make_matrix_covariates' works with valid inputs - not all numeric - no intercept", {
+test_that("'make_matrix_covariates' works with valid inputs - not all numeric - has intercept, has rows", {
+  set.seed(0)
+  data <- expand.grid(age = 0:9,
+                      region = c("a", "b"),
+                      sex = c("F", "M"))
+  data$popn <- rpois(n = nrow(data), lambda = 100)
+  data$deaths <- rpois(n = nrow(data), lambda = 10)
+  data$income <- runif(n = nrow(data))
+  formula <- ~ income * region
+  ans_obtained <- make_matrix_covariates(formula = formula, data = data, rows = 12)
+  data_scaled <- data
+  data_scaled$income <- as.numeric(scale(data_scaled$income))
+  ans_expected <- Matrix::sparse.model.matrix(~income*region,
+                                              data = data_scaled)[,-1]
+  rownames(ans_expected) <- NULL
+  ans_expected <- ans_expected[12, , drop = FALSE]
+  expect_identical(ans_obtained, ans_expected)
+})
+
+test_that("'make_matrix_covariates' works with valid inputs - not all numeric - no intercept, rows NULL", {
   set.seed(0)
   data <- expand.grid(age = 0:9,
                       region = c("a", "b"),
@@ -1364,7 +1368,7 @@ test_that("'make_matrix_covariates' works with valid inputs - not all numeric -
   data$deaths <- rpois(n = nrow(data), lambda = 10)
   data$income <- runif(n = nrow(data))
   formula <- ~ income * region - 1
-  ans_obtained <- make_matrix_covariates(formula = formula, data = data)
+  ans_obtained <- make_matrix_covariates(formula = formula, data = data, rows = NULL)
   data_scaled <- data
   data_scaled$income <- as.numeric(scale(data_scaled$income))
   ans_expected <- Matrix::sparse.model.matrix(~income*region,
@@ -1398,15 +1402,6 @@ test_that("'make_offset' works with valid inputs - has NA", {
     expect_identical(ans_obtained, ans_expected)
 })
 
-test_that("'make_offset' works with valid inputs - no NA", {
-    data <- expand.grid(age = 0:2, time = 2000:2001, sex = 1:2)
-    data$wt <- seq_len(nrow(data))
-    ans_obtained <- make_offset(nm_offset_data = "~ wt + 1",
-                                data = data)
-    ans_expected <- as.double(data$wt) + 1
-    expect_identical(ans_obtained, ans_expected)
-})
-
 
 ## 'make_offset_ones' -----------------------------------------------------
 
@@ -1455,13 +1450,13 @@ test_that("'make_outcome' works with valid inputs", {
     expect_identical(ans_obtained, ans_expected)
 })
 
-test_that("'make_outcome' throws error when variable not found", {
-    data <- expand.grid(age = 0:2, time = 2000:2001, sex = 1:2)
-    data$deaths <- seq_len(nrow(data))
-    formula <- wrong ~ age:sex + time
-    expect_error(make_outcome(formula = formula,
-                              data = data),
-                 "Internal error: response \"wrong\" not found in `data`.")
+test_that("'make_outcome' returns NULL when variable not found", {
+  data <- expand.grid(age = 0:2, time = 2000:2001, sex = 1:2)
+  data$deaths <- seq_len(nrow(data))
+  formula <- not_in_data ~ age:sex + time
+  expect_identical(make_outcome(formula = formula,
+                                 data = data),
+                    NULL)
 })
 
 
@@ -1503,7 +1498,7 @@ test_that("'make_outcome_offset_matrices' works with model without offset", {
   formula <- deaths ~ age * sex + region
   mod <- mod_pois(formula = formula,
                   data = data,
-                  exposure = 1)
+                  exposure = NULL)
   ans_obtained <- make_outcome_offset_matrices(mod, aggregate = TRUE)
   data_ag <- aggregate(data["deaths"], data[c("age", "sex", "region")], sum)
   ans_expected <- list(outcome = data_ag[["deaths"]],
@@ -1554,7 +1549,7 @@ test_that("'make_outcome_offset_matrices' works with model with offset", {
   formula <- deaths ~ age * sex + region
   mod <- mod_pois(formula = formula,
                   data = data,
-                  exposure = 1)
+                  exposure = NULL)
   ans_obtained <- make_outcome_offset_matrices(mod, aggregate = FALSE)
   ans_expected <- list(outcome = mod$outcome[-1],
                        offset = mod$offset[-1],
@@ -1667,6 +1662,80 @@ test_that("'make_priors' works with valid inputs - has intercept", {
 })
 
 
+## 'make_rows' ----------------------------------------------------------------
+
+test_that("NULL rows returns NULL", {
+  df <- data.frame(x = 1:3)
+  expect_null(make_rows(df, rlang::quo(NULL)))
+})
+
+test_that("logical rows works (including expression) and forbids NA", {
+  df <- data.frame(x = 1:4)
+  rows <- rlang::quo(c(TRUE, FALSE, TRUE, FALSE))
+  expect_identical(make_rows(df, rows), c(1L, 3L))
+  expect_identical(make_rows(df, rlang::quo(TRUE)), 1:4)           # recycled
+  expect_identical(make_rows(df, rlang::quo(x > 2)), c(3L, 4L))    # expression
+  expect_error(make_rows(df, rlang::quo(c(TRUE, NA, FALSE, TRUE))),
+               "`rows` has NA")
+})
+
+test_that("logical rows must recycle to length 1 or nrow(data)", {
+  df <- data.frame(x = 1:4)
+  expect_error(make_rows(df, rlang::quo(c(TRUE, FALSE))),
+               class = "vctrs_error_recycle_incompatible_size")
+})
+
+test_that("numeric rows works (including expression) and forbids NA/0", {
+  df <- data.frame(x = 1:5)
+  expect_identical(make_rows(df, rlang::quo(c(1, 3, 5))), c(1L, 3L, 5L))
+  expect_identical(make_rows(df, rlang::quo(2.0)), 2L)               # length-1 ok
+  expect_identical(make_rows(df, rlang::quo(which(x >= 4))), c(4L, 5L))  # expression
+  expect_error(make_rows(df, rlang::quo(c(1, NA))),
+               "`rows` has NA")
+  expect_error(make_rows(df, rlang::quo(c(1, 0))),
+               "must not contain 0")
+})
+
+test_that("numeric rows disallow duplicates", {
+  df <- data.frame(x = 1:5)
+  expect_error(make_rows(df, rlang::quo(c(1, 1))),
+               "duplicated indices")
+  expect_error(make_rows(df, rlang::quo(c(-2, -2))),
+               "duplicated indices")
+})
+
+test_that("numeric rows disallow mixing positive and negative", {
+  df <- data.frame(x = 1:5)
+  expect_error(make_rows(df, rlang::quo(c(1, -2))),
+               "`rows` mixes positive and negative row numbers.")
+})
+
+test_that("numeric rows error on out-of-bounds", {
+  df <- data.frame(x = 1:5)
+  expect_error(make_rows(df, rlang::quo(6)),
+               "`rows` has invalid row numbers")
+  expect_error(make_rows(df, rlang::quo(-6)),
+               "`rows` has invalid row numbers")
+})
+
+test_that("negative numeric rows drop those rows", {
+  df <- data.frame(x = 1:5)
+  expect_identical(make_rows(df, rlang::quo(-c(2, 4))), c(1L, 3L, 5L))
+})
+
+test_that("rows must evaluate to a vector", {
+  df <- data.frame(x = 1:3)
+  expect_error(make_rows(df, rlang::quo(df)),
+               "must evaluate to an atomic vector")
+})
+
+test_that("rows must be logical or numeric", {
+  df <- data.frame(x = 1:3)
+  expect_error(make_rows(df, rlang::quo(letters)),
+               "`rows` must evaluate to a logical vector or a numeric index vector")
+})
+
+
 ## 'make_seed' --------------------------------------------------------------
 
 test_that("'make_seed' returns a single unique integer", {
@@ -1806,7 +1875,7 @@ test_that("'make_use_term' works", {
                       deaths = 3)
   mod <- mod_pois(deaths ~ age * sex + age * region + sex * time,
                   data = data,
-                  exposure = 1)
+                  exposure = NULL)
   vars_inner <- c("sex", "age")
   ans_obtained <- make_use_term(mod = mod, vars_inner = vars_inner)
   ans_expected <- c(T, T, T, F, F, T, F, F)
@@ -1823,7 +1892,7 @@ test_that("'make_use_term' throws correct error when 'vars_inner' has invalid va
                       deaths = 3)
   mod <- mod_pois(deaths ~ age * sex + age * region + sex * time,
                   data = data,
-                  exposure = 1)
+                  exposure = NULL)
   vars_inner <- c("sex", "wrong")
   expect_error(make_use_term(mod = mod, vars_inner = vars_inner),
                "`vars_inner` has variable not found in model.")
@@ -1838,7 +1907,7 @@ test_that("'make_use_term' throws correct error when cannot form term from 'vars
                       deaths = 3)
   mod <- mod_pois(deaths ~ age : sex + age : region + sex * time,
                   data = data,
-                  exposure = 1)
+                  exposure = NULL)
   vars_inner <- "age"
   expect_error(make_use_term(mod = mod, vars_inner = vars_inner),
                "No terms in model can be formed from `vars_inner`.")
@@ -1853,7 +1922,7 @@ test_that("'make_use_term' throws correct error when can form all terms from 'va
                       deaths = 3)
   mod <- mod_pois(deaths ~ age : sex + age : region + sex * time,
                   data = data,
-                  exposure = 1)
+                  exposure = NULL)
   vars_inner <- c("age", "sex", "region", "time")
   expect_error(make_use_term(mod = mod, vars_inner = vars_inner),
                "All terms in model can be formed from `vars_inner`.")
@@ -2055,8 +2124,6 @@ test_that("'message_suspicious_rates' returns message with 2 rows", {
 })
 
 
-
-
 ## 'n_col' --------------------------------------------------------------------
 
 test_that("'n_col' works with ordinary matrix", {
@@ -2070,27 +2137,6 @@ test_that("'n_col' works with Matrix matrix", {
 })
 
 
-
-## 'n_comp_svd' ---------------------------------------------------------------
-
-test_that("'n_comp_svd' works when no 'n' supplied", {
-  ans_obtained <- n_comp_svd(n_comp = NULL, nm_n_comp = "n", ssvd = HMD)
-  ans_expected <- 3L
-  expect_identical(ans_obtained, ans_expected)
-})
-
-test_that("'n_comp_svd' works when valid 'n' supplied", {
-  ans_obtained <- n_comp_svd(n_comp = 3, nm_n_comp = "n", ssvd = HMD)
-  ans_expected <- 3L
-  expect_identical(ans_obtained, ans_expected)
-})
-
-test_that("'n_comp_svd' throws correct error when n is too high", {
-  expect_error(n_comp_svd(n_comp = 11, nm_n_comp = "n_component", ssvd = HMD),
-               "`n_component` larger than number of components of `ssvd`.")
-})
-
-
 ## 'print_prior' -------------------------------------------------------
 
 test_that("'print_prior' works", {
@@ -2202,7 +2248,7 @@ test_that("'set_priors_known' works with valid inputs", {
 })
 
 
-## 'str_call_args_along' ---------------------------------------------------------
+## 'str_call_args_along' ------------------------------------------------------
 
 test_that("'str_call_args_along' works - no along", {
   prior <- RW()
@@ -2236,6 +2282,20 @@ test_that("'str_call_args_svd' works - AR", {
 })
 
 
+## 'str_call_args_coef_drw' ---------------------------------------------------
+
+test_that("'str_call_args_coef_drw' works", {
+  prior <- DRW()
+  ans_obtained <- str_call_args_coef_drw(prior)
+  ans_expected <- c("", "", "", "")
+  expect_identical(ans_obtained, ans_expected)
+  prior <- DRW(min = 0.9, shape2 = 3)
+  ans_obtained <- str_call_args_coef_drw(prior)
+  ans_expected <- c("", "shape2=3", "min=0.9", "")
+  expect_identical(ans_obtained, ans_expected)
+})
+
+
 ## 'str_call_args_con' --------------------------------------------------------
 
 test_that("'str_call_args_con' works - con is 'none'", {
@@ -2287,10 +2347,10 @@ test_that("'str_call_args_n_comp' works - n_comp provided", {
 })
 
 
-## 'str_call_args_n_seas' --------------------------------------------------------
+## 'str_call_args_n_seas' -----------------------------------------------------
 
 test_that("'str_call_args_n_seas' works", {
-  prior <- RW_Seas(n_seas=3)
+  prior <- RW_Seas(n_seas=3, s_seas = 0)
   ans_obtained <- str_call_args_n_seas(prior)
   ans_expected <- "n_seas=3"
   expect_identical(ans_obtained, ans_expected)
@@ -2327,14 +2387,14 @@ test_that("'str_call_args_scale' works - scale not 1", {
 ## 'str_call_args_sd' -----------------------------------------------------
 
 test_that("'str_call_args_sd' works - sd = 1", {
-  prior <- RW_Seas(n=3)
+  prior <- RW_Seas(n=3,s_seas=1)
   ans_obtained <- str_call_args_sd(prior)
   ans_expected <- ""
   expect_identical(ans_obtained, ans_expected)
 })
 
 test_that("'str_call_args_sd' works - sd not 1", {
-  prior <- RW_Seas(n=2,sd = 0.3)
+  prior <- RW_Seas(n=2,s_seas=0,sd = 0.3)
   ans_obtained <- str_call_args_sd(prior)
   ans_expected <- "sd=0.3"
   expect_identical(ans_obtained, ans_expected)
@@ -2344,14 +2404,14 @@ test_that("'str_call_args_sd' works - sd not 1", {
 ## 'str_call_args_sd_seas' -----------------------------------------------------
 
 test_that("'str_call_args_sd_seas' works - sd_seas = 1", {
-  prior <- RW_Seas(n=3)
+  prior <- RW_Seas(n=3,s_seas=0)
   ans_obtained <- str_call_args_sd_seas(prior)
   ans_expected <- ""
   expect_identical(ans_obtained, ans_expected)
 })
 
 test_that("'str_call_args_sd_seas' works - sd_seas not 1", {
-  prior <- RW_Seas(n=2,sd_seas = 0.3)
+  prior <- RW_Seas(n=2,s_seas=0,sd_seas = 0.3)
   ans_obtained <- str_call_args_sd_seas(prior)
   ans_expected <- "sd_seas=0.3"
   expect_identical(ans_obtained, ans_expected)
@@ -2399,22 +2459,20 @@ test_that("'str_call_args_svd' works - joint", {
 })
 
 
-## 'to_factor' ----------------------------------------------------------------
+## 'user_specified_offset' ----------------------------------------------------
 
-test_that("'to_factor' leaves existing factor unchanged", {
-  x <- factor(letters)
-  expect_identical(to_factor(x), x)
+test_that("'user_specified_offset' works with pois", {
+  expect_true(user_specified_offset(make_small_mod_pois(use_exposure = TRUE)))
+  expect_false(user_specified_offset(make_small_mod_pois(use_exposure = FALSE)))
 })
 
-test_that("'to_factor' orders numeric x by values", {
-  x <- c(3, 1, 0.2, 1)
-  expect_identical(to_factor(x), factor(x, levels = c(0.2, 1, 3)))
+test_that("'user_specified_offset' works with binom", {
+  expect_true(user_specified_offset(make_small_mod_binom()))
 })
 
-
-test_that("'to_factor' orders non-numeric non-factor by order of appearance", {
-  x <- c("b", "a", 1, "a")
-  expect_identical(to_factor(x), factor(x, levels = c("b", "a", 1)))
+test_that("'user_specified_offset' works with norm", {
+  expect_true(user_specified_offset(make_small_mod_norm(use_weights = TRUE)))
+  expect_false(user_specified_offset(make_small_mod_norm(use_weights = FALSE)))
 })
 
 
