@@ -47,10 +47,12 @@ test_that("'make_dim_svd' works - total", {
   prior <- SVD(LFP)
   dimnames_term <- list(region = c("A", "B"),
                         age = poputils::age_labels(type = "five", min = 15, max = 60))
+  var_time <- "time"
   var_sexgender <- NULL
   var_age <- "age"
   ans_obtained <- make_dim_svd(prior,
                                dimnames_term = dimnames_term,
+                               var_time = var_time,
                                var_sexgender = var_sexgender,
                                var_age = var_age)
   ans_expected <- c(3L, region = 2L)
@@ -64,8 +66,10 @@ test_that("'make_dim_svd' works - indep", {
                         age = poputils::age_labels(type = "five", min = 15, max = 60))
   var_sexgender <- "sex"
   var_age <- "age"
+  var_time <- "time"
   ans_obtained <- make_dim_svd(prior,
                                dimnames_term = dimnames_term,
+                               var_time = var_time,
                                var_sexgender = var_sexgender,
                                var_age = var_age)
   ans_expected <- c(6L, region = 2L)
@@ -79,14 +83,52 @@ test_that("'make_dim_svd' works - joint", {
                         age = poputils::age_labels(type = "five", min = 15, max = 60))
   var_sexgender <- "sex"
   var_age <- "age"
+  var_time <- "time"
   ans_obtained <- make_dim_svd(prior,
                                dimnames_term = dimnames_term,
+                               var_time = var_time,
                                var_sexgender = var_sexgender,
                                var_age = var_age)
   ans_expected <- c(3L, region = 2L)
   expect_identical(ans_obtained, ans_expected)
 })
 
+test_that("'make_dim_svd' works - total, con = 'by'", {
+  prior <- SVD_RW(LFP, n_comp = 3, con = "by")
+  dimnames_term <- list(region = c("A", "B", "C"),
+                        time = 2001:2005,
+                        age = poputils::age_labels(type = "five", min = 15, max = 60))
+  var_time <- "time"
+  var_sexgender <- NULL
+  var_age <- "age"
+  ans_obtained <- make_dim_svd(prior,
+                               dimnames_term = dimnames_term,
+                               var_time = var_time,
+                               var_sexgender = var_sexgender,
+                               var_age = var_age)
+  ans_expected <- c(3L, region = 2L, time = 5L)
+  expect_identical(ans_obtained, ans_expected)
+})
+
+test_that("'make_dim_svd' works - total, con = 'non'", {
+  prior <- SVD_RW(LFP, n_comp = 3)
+  dimnames_term <- list(region = c("A", "B", "C"),
+                        time = 2001:2005,
+                        age = poputils::age_labels(type = "five", min = 15, max = 60))
+  var_time <- "time"
+  var_sexgender <- NULL
+  var_age <- "age"
+  ans_obtained <- make_dim_svd(prior,
+                               dimnames_term = dimnames_term,
+                               var_time = var_time,
+                               var_sexgender = var_sexgender,
+                               var_age = var_age)
+  ans_expected <- c(3L, region = 3L, time = 5L)
+  expect_identical(ans_obtained, ans_expected)
+})
+
+
+
 
 ## 'make_i_along_agetime' -----------------------------------------------------
 
@@ -254,6 +296,37 @@ test_that("'make_i_along_inner' throws correct error when 'along' not found", {
 })
 
 
+## 'make_i_time_svddynamic' ------------------------------------------
+
+test_that("'make_i_time_svddynamic' works - age, sex, time", {
+  prior <- SVD_AR(HMD, n_comp = 4)
+  ans_obtained <- make_i_time_svddynamic(prior = prior,
+                                         dimnames_term = list(sex = c("f", "m"),
+                                                              age = c(0:99,
+                                                                      "100+"),
+                                                              time = 1:10),
+                                         var_time = "time",
+                                         var_age = "age",
+                                         var_sexgender = "sex")
+  ans_expected <- 2L
+  expect_identical(ans_obtained, ans_expected)
+})
+
+test_that("'make_i_time_svddynamic' works - age, region, time", {
+  prior <- SVD_AR(HMD, n_comp = 4)
+  ans_obtained <- make_i_time_svddynamic(prior = prior,
+                                         dimnames_term = list(region = c("f", "m"),
+                                                              age = c(0:99,
+                                                                      "100+"),
+                                                              time = 1:10),
+                                         var_time = "time",
+                                         var_age = "age",
+                                         var_sexgender = "sex")
+  ans_expected <- 3L
+  expect_identical(ans_obtained, ans_expected)
+})
+
+
 ## 'make_matrix_along_by' -----------------------------------------------------
 
 test_that("'make_matrix_along_by' works when 'i_along' is 1", {
@@ -615,7 +688,8 @@ test_that("'make_matrix_draws_svd_appendzero' works - age and time", {
                                                    var_sexgender = "sex")
   ans_expected <- Matrix::kronecker(rbind(0, Matrix::.sparseDiagonal(4)),
                                     Matrix::.sparseDiagonal(3))
-  expect_identical(ans_obtained, ans_expected)
+  expect_identical(unname(as.matrix(ans_obtained)),
+                   unname(as.matrix(ans_expected)))
 })
 
 test_that("'make_matrix_draws_svd_appendzero' works - time and age", {
@@ -629,7 +703,8 @@ test_that("'make_matrix_draws_svd_appendzero' works - time and age", {
                                                    var_sexgender = "sex")
   ans_expected <- Matrix::kronecker(rbind(0, Matrix::.sparseDiagonal(4)),
                                     Matrix::.sparseDiagonal(3))
-  expect_identical(ans_obtained, ans_expected)
+  expect_identical(unname(as.matrix(ans_obtained)),
+                   unname(as.matrix(ans_expected)))
 })
 
 
@@ -814,13 +889,13 @@ test_that("'make_matrix_effectfree_effect_inner' works - append column and appen
   expect_equal(colSums(ans), rep(0, 3))
 })
 
-test_that("'make_matrix_effectfree_effect' works with SVD-based matrix_sub_orig", {
+test_that("'make_matrix_effectfree_effect_inner' works with SVD-based matrix_sub_orig", {
   dimnames_term <- list(age = poputils::age_labels(type = "five", max = 60),
                         time = 2001:2005)
   var_time <- "time"
   var_age <- "age"
   var_sexgender <- "sex"
-  prior <- SVD_RW(HMD)
+  prior <- SVD_RW(HMD, n_comp = 3)
   ans <- make_matrix_effectfree_effect_inner(prior = prior,
                                              dimnames_term = dimnames_term,
                                              var_time = var_time,
@@ -830,13 +905,13 @@ test_that("'make_matrix_effectfree_effect' works with SVD-based matrix_sub_orig"
   expect_equal(dim(ans), c(5 * 13, 4 * 3))
 })
 
-test_that("'make_matrix_effectfree_effect' works with SVD-based matrix_sub_orig, con is 'by'", {
+test_that("'make_matrix_effectfree_effect_inner' works with SVD-based matrix_sub_orig, con is 'by'", {
   dimnames_term <- list(age = poputils::age_labels(type = "five", max = 60),
                         time = 2001:2005)
   var_time <- "time"
   var_age <- "age"
   var_sexgender <- "sex"
-  prior <- SVD_RW(HMD, con = "by")
+  prior <- SVD_RW(HMD, con = "by", n_comp = 3)
   ans <- make_matrix_effectfree_effect_inner(prior = prior,
                                              dimnames_term = dimnames_term,
                                              var_time = var_time,
@@ -858,7 +933,8 @@ test_that("'make_matrix_perm_agesex_from_front' works - age and sex", {
                                           dim_after = dim_after)
   a_orig <- array(1:40, dim = c(4, 5, 2))
   a_perm <- aperm(a_orig, perm = c(2, 3, 1))
-  expect_identical(as.integer(m %*% as.integer(a_perm)), as.integer(a_orig))
+  expect_identical(as.integer(as.matrix(m) %*% as.numeric(as.matrix(a_perm))),
+                   as.integer(as.matrix(a_orig)))
 })
 
 test_that("'make_matrix_perm_agesex_from_front' works - just age", {
@@ -870,7 +946,8 @@ test_that("'make_matrix_perm_agesex_from_front' works - just age", {
                                           dim_after = dim_after)
   a_orig <- array(1:20, dim = c(4, 5))
   a_perm <- t(a_orig)
-  expect_identical(as.integer(m %*% as.integer(a_perm)), as.integer(a_orig))
+  expect_identical(as.integer(as.matrix(m) %*% as.numeric(as.matrix(a_perm))),
+                   as.integer(a_orig))
 })
 
 test_that("'make_matrix_perm_agesex_from_front' works - just age, age first", {
@@ -880,7 +957,7 @@ test_that("'make_matrix_perm_agesex_from_front' works - just age, age first", {
   m <- make_matrix_perm_agesex_from_front(i_age = i_age,
                                           i_sexgender = i_sexgender,
                                           dim_after = dim_after)
-  expect_identical(as.matrix(m), diag(20))
+  expect_identical(unname(as.matrix(m)), diag(20))
 })
 
 
@@ -889,13 +966,13 @@ test_that("'make_matrix_perm_agesex_from_front' works - just age, age first", {
 test_that("'make_matrix_perm_along_from_front' reverses effects of 'make_matrix_perm_along_to_front'", {
   m1 <- make_matrix_perm_along_to_front(dim_after = c(4L, 2:3), i_along = 3L)
   m2 <- make_matrix_perm_along_from_front(dim_after = 2:4, i_along = 3L)
-  expect_identical(as.matrix(m2 %*% m1), diag(24L))
+  expect_identical(unname(as.matrix(m2 %*% m1)), diag(24L))
   m1 <- make_matrix_perm_along_to_front(dim_after = c(3L, 2L, 4L), i_along = 2L)
   m2 <- make_matrix_perm_along_from_front(dim_after = 2:4, i_along = 2L)
-  expect_identical(as.matrix(m2 %*% m1), diag(24L))
+  expect_identical(unname(as.matrix(m2 %*% m1)), diag(24L))
   m1 <- make_matrix_perm_along_to_front(dim_after = 2:4, i_along = 1L)
   m2 <- make_matrix_perm_along_from_front(dim_after = 2:4, i_along = 1L)
-  expect_identical(as.matrix(m2 %*% m1), diag(24L))
+  expect_identical(unname(as.matrix(m2 %*% m1)), diag(24L))
 })
 
 
@@ -909,6 +986,7 @@ test_that("'make_matrix_perm_along_to_front' works with 2 dimensions, along firs
 
 test_that("'make_matrix_perm_along_to_front' works with 2 dimensions, along second", {
   m <- make_matrix_perm_along_to_front(i_along = 2L, dim_after = 2:3)
+  m <- as.matrix(m)
   x <- matrix(1:6, nr = 3)
   expect_identical(matrix(m %*% as.integer(x), nr = 2),
                    1 * t(x))
@@ -917,6 +995,7 @@ test_that("'make_matrix_perm_along_to_front' works with 2 dimensions, along seco
 
 test_that("'make_matrix_perm_along_to_front' works with 3 dimensions, along second", {
   m <- make_matrix_perm_along_to_front(i_along = 2L, dim_after = c(2L, 3L, 2L))
+  m <- as.matrix(m)
   x <- array(1:12, dim = c(3, 2, 2))
   expect_identical(array(m %*% as.integer(x), dim = c(2, 3, 2)),
                    1 * aperm(x, perm = c(2, 1, 3)))
@@ -937,7 +1016,7 @@ test_that("'make_matrix_spline' works", {
 ## 'make_matrix_sub_orig_svd' -------------------------------------------------
 
 test_that("'make_matrix_sub_orig_svd' works with bage_prior_svd_ar - time x age interaction, con is 'none'", {
-  prior <- SVD_RW(HMD)
+  prior <- SVD_RW(HMD, n_comp = 3)
   dimnames_term <- list(time = 2001:2005,
                         age = poputils::age_labels(type = "five", max = 60))
   var_age <- "age"
@@ -953,7 +1032,7 @@ test_that("'make_matrix_sub_orig_svd' works with bage_prior_svd_ar - time x age
 })
 
 test_that("'make_matrix_sub_orig_svd' works with bage_prior_svd_ar - time x age interaction, con is 'none'", {
-  prior <- SVD_RW(HMD)
+  prior <- SVD_RW(HMD, n_comp = 3)
   dimnames_term <- list(time = 2001:2005,
                         age = poputils::age_labels(type = "five", max = 60))
   var_age <- "age"
@@ -969,7 +1048,7 @@ test_that("'make_matrix_sub_orig_svd' works with bage_prior_svd_ar - time x age
 })
 
 test_that("'make_matrix_sub_orig_svd' works with bage_prior_svd_ar - sex x time x age interaction, con is 'none'", {
-  prior <- SVD_RW(HMD)
+  prior <- SVD_RW(HMD, n_comp = 3)
   dimnames_term <- list(sex = c("f", "m"),
                         time = 2001:2005,
                         age = poputils::age_labels(type = "five", max = 60))
